#include <iostream>
#include "model.h"
#include "error.h"
#include "data.h"
#include <QPoint>
#include <stdlib.h>

namespace Insert {

/** Temporary data **/
Model* _currModel = new Model;

/** Models **/

void closeModel() {
	Data::insertModel(_currModel);
	_currModel = new Model;
}

/** Objects **/

Object* getObject(const vector<string>& name) {
	Object* obj = _currModel->object(name);
	if(!obj) {
		obj = new Object(name);
//cerr << "Made a new object with name ";
//for(int n = 0; n < name.size(); ++n) cerr << name[n] << ' ';
//cerr << endl;
		_currModel->addObj(obj);
	}
	return obj;
}

/** Positions **/

void xposition(const vector<string>& args) {
	if(args.empty()) {
		Error::too_few_args("xpos",args);
		return;
	}

	// extract the name
	vector<string> name;
	for(int n = 0; n < args.size() - 1; ++n) name.push_back(args[n]);
	Object* obj = getObject(name);

	// extract the coordinate
	int x = atoi((args[args.size()-1]).c_str());
	if(x == 0 && args[args.size()-1] != "0") {
		Error::int_expected("xpos",args,args.size());
		return;
	}

	obj->set_xpos(x);
}

void xpost(const vector<string>& args) {
	if(args.size() < 2) {
		Error::too_few_args("xpos_t",args);
		return;
	}

	// extract the time 
	int t = atoi((args[0]).c_str());
	if(t == 0 && args[0] != "0") {
		Error::int_expected("xpos_t",args,1);
		return;
	}

	// extract the name
	vector<string> name;
	for(int n = 1; n < args.size() - 1; ++n) name.push_back(args[n]);
	Object* obj = getObject(name);

	// extract the coordinate
	int x = atoi((args[args.size()-1]).c_str());
	if(x == 0 && args[args.size()-1] != "0") {
		Error::int_expected("xpos_t",args,args.size());
		return;
	}

	obj->set_xpos(x,t);

}

void yposition(const vector<string>& args) {
	if(args.empty()) {
		Error::too_few_args("ypos",args);
		return;
	}

	// extract the name
	vector<string> name;
	for(int n = 0; n < args.size() - 1; ++n) name.push_back(args[n]);
	Object* obj = getObject(name);

	// extract the coordinate
	int y = atoi((args[args.size()-1]).c_str());
	if(y == 0 && args[args.size()-1] != "0") {
		Error::int_expected("ypos",args,args.size());
		return;
	}

	obj->set_ypos(y);
}

void ypost(const vector<string>& args) {
	if(args.size() < 2) {
		Error::too_few_args("ypos_t",args);
		return;
	}

	// extract the time 
	int t = atoi((args[0]).c_str());
	if(t == 0 && args[0] != "0") {
		Error::int_expected("ypos_t",args,1);
		return;
	}

	// extract the name
	vector<string> name;
	for(int n = 1; n < args.size() - 1; ++n) name.push_back(args[n]);
	Object* obj = getObject(name);

	// extract the coordinate
	int y = atoi((args[args.size()-1]).c_str());
	if(y == 0 && args[args.size()-1] != "0") {
		Error::int_expected("ypos_t",args,args.size());
		return;
	}

	obj->set_ypos(y,t);

}

void depth(const vector<string>& args) {
	if(args.empty()) {
		Error::too_few_args("depth",args);
		return;
	}

	// extract the name
	vector<string> name;
	for(int n = 0; n < args.size() - 1; ++n) name.push_back(args[n]);
	Object* obj = getObject(name);

	// extract the coordinate
	int d = atoi((args[args.size()-1]).c_str());
	if(d == 0 && args[args.size()-1] != "0") {
		Error::int_expected("depth",args,args.size());
		return;
	}

	obj->set_depth(d);
}

void deptht(const vector<string>& args) {
	if(args.size() < 2) {
		Error::too_few_args("deptht",args);
		return;
	}

	// extract the time 
	int t = atoi((args[0]).c_str());
	if(t == 0 && args[0] != "0") {
		Error::int_expected("ypos_t",args,1);
		return;
	}

	// extract the name
	vector<string> name;
	for(int n = 1; n < args.size() - 1; ++n) name.push_back(args[n]);
	Object* obj = getObject(name);

	// extract the coordinate
	int d = atoi((args[args.size()-1]).c_str());
	if(d == 0 && args[args.size()-1] != "0") {
		Error::int_expected("deptht",args,args.size());
		return;
	}

	obj->set_depth(d,t);
}


/** Text **/

void text(const vector<string>& args) {
	if(args.empty()) {
		Error::too_few_args("text",args);
		return;
	}

	// extract the name
	vector<string> name;
	for(int n = 0; n < args.size() - 1; ++n) name.push_back(args[n]);
	Object* obj = getObject(name);

	obj->set_text(args[args.size()-1]);
}

void textt(const vector<string>& args) {
	if(args.size() < 2) {
		Error::too_few_args("textt",args);
		return;
	}

	// extract the time 
	int t = atoi((args[0]).c_str());
	if(t == 0 && args[0] != "0") {
		Error::int_expected("ypos_t",args,1);
		return;
	}

	// extract the name
	vector<string> name;
	for(int n = 1; n < args.size() - 1; ++n) name.push_back(args[n]);
	Object* obj = getObject(name);

	obj->set_text(args[args.size()-1],t);
}

/** Colors **/

void color(const vector<string>& args) {
	if(args.size() < 3) {
		Error::too_few_args("color",args);
		return;
	}

	// extract the name
	vector<string> name;
	for(int n = 0; n < args.size() - 3; ++n) name.push_back(args[n]);
	Object* obj = getObject(name);

	// extract the color
	vector<int> rgb;
	for(int n = args.size() - 3; n < args.size(); ++n) {
		int c = atoi(args[n].c_str());
		if(c == 0 && args[n] != "0") Error::int_expected("color",args,n+1);
		rgb.push_back(c);
	}
	obj->set_color(rgb[0],rgb[1],rgb[2]);
}

void colort(const vector<string>& args) {
	if(args.size() < 4) {
		Error::too_few_args("color",args);
		return;
	}

	// extract the time 
	int t = atoi((args[0]).c_str());
	if(t == 0 && args[0] != "0") {
		Error::int_expected("color_t",args,1);
		return;
	}

	// extract the name
	vector<string> name;
	for(int n = 1; n < args.size() - 3; ++n) name.push_back(args[n]);
	Object* obj = getObject(name);

	// extract the color
	vector<int> rgb;
	for(int n = args.size() - 3; n < args.size(); ++n) {
		int c = atoi(args[n].c_str());
		if(c == 0 && args[n] != "0") Error::int_expected("color",args,n+1);
		rgb.push_back(c);
	}
	obj->set_color(rgb[0],rgb[1],rgb[2],t);
}

void pencolor(const vector<string>& args) {
	if(args.size() < 3) {
		Error::too_few_args("color",args);
		return;
	}

	// extract the name
	vector<string> name;
	for(int n = 0; n < args.size() - 3; ++n) name.push_back(args[n]);
	Object* obj = getObject(name);

	// extract the color
	vector<int> rgb;
	for(int n = args.size() - 3; n < args.size(); ++n) {
		int c = atoi(args[n].c_str());
		if(c == 0 && args[n] != "0") Error::int_expected("pencolor",args,n+1);
		rgb.push_back(c);
	}
	obj->set_pencolor(rgb[0],rgb[1],rgb[2]);
}

void pencolort(const vector<string>& args) {
	if(args.size() < 4) {
		Error::too_few_args("colort",args);
		return;
	}

	// extract the time 
	int t = atoi((args[0]).c_str());
	if(t == 0 && args[0] != "0") {
		Error::int_expected("pencolor_t",args,1);
		return;
	}

	// extract the name
	vector<string> name;
	for(int n = 1; n < args.size() - 3; ++n) name.push_back(args[n]);
	Object* obj = getObject(name);

	// extract the color
	vector<int> rgb;
	for(int n = args.size() - 3; n < args.size(); ++n) {
		int c = atoi(args[n].c_str());
		if(c == 0 && args[n] != "0") Error::int_expected("pencolor",args,n+1);
		rgb.push_back(c);
	}
	obj->set_pencolor(rgb[0],rgb[1],rgb[2],t);
}

/** Shapes **/

void polygon(const vector<string>& args) {
	if(args.empty()) {
		Error::too_few_args("polygon",args);
		return;
	}

	int np = atoi(args[0].c_str()) * 2;
	if(np == 0 && args[0] != "0") {
		Error::int_expected("polygon",args,1);
		return;
	}

	if(args.size() < (np  + 1)) {
		Error::too_few_args("polygon",args);
		return;
	}

	// extract the name
	vector<string> name;
	for(int n = 1; n < args.size() - np; ++n) name.push_back(args[n]);
	Object* obj = getObject(name);

	// extract the points
	vector<QPoint> poly;
	for(int n = args.size() - np; n < args.size(); ++n) {
		int x = atoi(args[n].c_str());
		if(x == 0 && args[n] != "0") Error::int_expected("polygon",args,n+1);
		++n;
		int y = atoi(args[n].c_str());
		if(y == 0 && args[n] != "0") Error::int_expected("polygon",args,n+1);
		poly.push_back(QPoint(x,y));
	}
	PolyShape* ps = new PolyShape(poly);
	obj->set_shape(ps);
}

void polygont(const vector<string>& args) {
	if(args.size() < 2) {
		Error::too_few_args("polygont",args);
		return;
	}

	// extract the time 
	int t = atoi((args[0]).c_str());
	if(t == 0 && args[0] != "0") {
		Error::int_expected("polygon_t",args,1);
		return;
	}

	// extract the number of vertices
	int np = atoi(args[1].c_str()) * 2;
	if(np == 0 && args[1] != "0") {
		Error::int_expected("polygont",args,2);
		return;
	}

	if(args.size() < (np  + 2)) {
		Error::too_few_args("polygont",args);
		return;
	}

	// extract the name
	vector<string> name;
	for(int n = 2; n < args.size() - np; ++n) name.push_back(args[n]);
	Object* obj = getObject(name);

	// extract the points
	vector<QPoint> poly;
	for(int n = args.size() - np; n < args.size(); ++n) {
		int x = atoi(args[n].c_str());
		if(x == 0 && args[n] != "0") Error::int_expected("polygont",args,n+1);
		++n;
		int y = atoi(args[n].c_str());
		if(y == 0 && args[n] != "0") Error::int_expected("polygont",args,n+1);
		poly.push_back(QPoint(x,y));
	}
	PolyShape* ps = new PolyShape(poly);
	obj->set_shape(ps,t);
}


void ellipse(const vector<string>& args) {
	if(args.size() < 2) {
		Error::too_few_args("ellipse",args);
		return;
	}

	// extract the name
	vector<string> name;
	for(int n = 0; n < args.size() - 2; ++n) name.push_back(args[n]);
	Object* obj = getObject(name);

	// extract the width and height
	int w = atoi((args[args.size()-2]).c_str());
	if(w == 0 && args[args.size()-2] != "0") Error::int_expected("ellipse",args,args.size()-1);
	int h = atoi((args[args.size()-1]).c_str());
	if(h == 0 && args[args.size()-1] != "0") Error::int_expected("ellipse",args,args.size());

	ElliShape* el = new ElliShape(w,h);
	obj->set_shape(el);
}

void ellipset(const vector<string>& args) {
	if(args.size() < 3) {
		Error::too_few_args("ellipse",args);
		return;
	}

	// extract the time 
	int t = atoi((args[0]).c_str());
	if(t == 0 && args[0] != "0") {
		Error::int_expected("ellipse_t",args,1);
		return;
	}

	// extract the name
	vector<string> name;
	for(int n = 1; n < args.size() - 2; ++n) name.push_back(args[n]);
	Object* obj = getObject(name);

	// extract the width and height
	int w = atoi((args[args.size()-2]).c_str());
	if(w == 0 && args[args.size()-2] != "0") Error::int_expected("ellipse",args,args.size()-1);
	int h = atoi((args[args.size()-1]).c_str());
	if(h == 0 && args[args.size()-1] != "0") Error::int_expected("ellipse",args,args.size());

	ElliShape* el = new ElliShape(w,h);
	obj->set_shape(el,t);
}

}
