#include <QtGui>
#include <iostream>
#include "mainwindow.h"
#include "data.h"
#include "error.h"

/** Construction of main window **/

MainWindow::MainWindow(QWidget* parent) : QMainWindow(parent) {
	createMenuBar();
	createToolBar();
	createTimer();
}

void MainWindow::createMenuBar() {
	
	// File menu
	QMenu* fileMenu = menuBar()->addMenu("&File");
	
	QAction* openAction = fileMenu->addAction("&Open");
	fileMenu->addSeparator();
	QAction*	exitAction = fileMenu->addAction("&Quit");

	connect(exitAction,SIGNAL(triggered()),qApp,SLOT(quit()));
	connect(openAction,SIGNAL(triggered()),this,SLOT(openFile()));

	// Options menu
	QMenu* optMenu = menuBar()->addMenu("&Options");

	QAction* fontAction = optMenu->addAction("Font");

	connect(fontAction,SIGNAL(triggered()),this,SLOT(font()));

	// Help menu
	QMenu* helpMenu = menuBar()->addMenu("&Help");
	
	QAction* aboutAction = helpMenu->addAction("About");

	connect(aboutAction,SIGNAL(triggered()),this,SLOT(about()));

	// Canvas
	_canvas = new Canvas();
	setCentralWidget(_canvas);

}

void MainWindow::createToolBar() {
	QToolBar* toolbar = new QToolBar();
	QAction* playB = toolbar->addAction("Play");
	QAction* stopB = toolbar->addAction("Stop");
	QAction* pauseB = toolbar->addAction("Pause");
	QAction* nextB = toolbar->addAction("Next");
	QAction* prevB = toolbar->addAction("Previous");
	addToolBar(Qt::BottomToolBarArea,toolbar);

	connect(nextB,SIGNAL(triggered()),this,SLOT(nextTime()));
	connect(prevB,SIGNAL(triggered()),this,SLOT(prevTime()));
	connect(playB,SIGNAL(triggered()),this,SLOT(play()));
	connect(stopB,SIGNAL(triggered()),this,SLOT(stop()));
	connect(pauseB,SIGNAL(triggered()),this,SLOT(pause()));
}

void MainWindow::createTimer() {
	_timer = new QTimer();
	connect(_timer,SIGNAL(timeout()),this,SLOT(nextTime()));
}

/** Open a file **/

extern FILE* lexin;
extern int parseparse();
extern void my_flush_buffer();

void MainWindow::loadFile(QString filename) {
	Data::clearModels();

	my_flush_buffer();
	Error::clearerrors();
	lexin = fopen(qPrintable(filename),"r");
	if(lexin) {
		parseparse();
		if(Error::nrerrors()) QMessageBox::critical(this,"Parse error","An error occurred during parsing");
	}
	_canvas->update();
}

void MainWindow::openFile() {
	Data::clearModels();

	QString fname = QFileDialog::getOpenFileName(this,"Open File","./","*");
	loadFile(fname);
}

/** Options **/

void MainWindow::font() {
	bool ok;
	QFont font = QFontDialog::getFont(&ok,Data::font(),this);
	if(ok) { 
		Data::setFont(font);
		_canvas->update();
	}
}

/** Play movie **/

void MainWindow::nextTime() {
	if(Data::nextTime()) _canvas->update();
	else _timer->stop();
}

void MainWindow::prevTime() {
	if(Data::prevTime()) _canvas->update();
}

void MainWindow::play() {
	_timer->start(250);
}

void MainWindow::stop() {
	_timer->stop();
	Data::resetTime();
	_canvas->update();
}

void MainWindow::pause() {
	_timer->stop();
}

/** About IDPDraw **/

void MainWindow::about() const {
	QMessageBox::about(0,"About IDPDraw","IDPDraw 0.1\nCopyright 2009  Johan Wittocx - K.U.Leuven");
}
