%{

#include <string>
#include <vector>
#include <iostream>
#include "insert.h"
#include "error.h"

using namespace std;

extern int lexlex();
int parselex() {
	return lexlex();
}

void parseerror(const char*);

%}

%error-verbose

/** Data types **/

%union{
	string* str;

	vector<string>*	vstr;
}

/** Keywords **/
%token POLYGON
%token ELLIPSE
%token XPOS
%token YPOS
%token COLOR
%token PENCOLOR
%token DEPTH
%token TEXT

%token POLYGON_T
%token ELLIPSE_T
%token XPOS_T
%token YPOS_T
%token COLOR_T
%token PENCOLOR_T
%token DEPTH_T
%token TEXT_T

/** Terminals **/
%token <str> TERM

/** Non-terminals with semantic value **/
%type	<str>		term
%type <vstr>	terms
%type <vstr>	common

%%

idpdraw		:	atomlist							{ Insert::closeModel();		}
				|  error
				;

atomlist		: /* empty */
				| atomlist atom
				| atomlist dynatom
				;

atom			: polygon
				| ellipse
				| xposition
				| yposition
				| color
				| pencolor
				| depth
				| text
				| otheratom
				;


dynatom		: polygont
				| ellipset
				| xpost
				| ypost
				| colort
				| pencolort
				| deptht
				| textt
				;

polygon		: POLYGON common		{ Insert::polygon(*$2); delete($2);		}
				;

ellipse		: ELLIPSE common		{ Insert::ellipse(*$2); delete($2);		}
				;

xposition	: XPOS common			{ Insert::xposition(*$2); delete($2);	}
				;

yposition	: YPOS common			{ Insert::yposition(*$2); delete($2);	}
				;

color			: COLOR common			{ Insert::color(*$2); delete($2);		}
				;
				
pencolor		: PENCOLOR common		{ Insert::pencolor(*$2); delete($2);	}
				;
				
depth			: DEPTH common			{ Insert::depth(*$2); delete($2);		}
				;

text			: TEXT common			{ Insert::text(*$2); delete($2);			}
				;

polygont		: POLYGON_T common	{ Insert::polygont(*$2); delete($2);	}
				;

ellipset		: ELLIPSE_T common	{ Insert::ellipset(*$2); delete($2);	}
				;

xpost			: XPOS_T common		{ Insert::xpost(*$2); delete($2);		}
				;

ypost			: YPOS_T common		{ Insert::ypost(*$2); delete($2);		}
				;

colort		: COLOR_T common		{ Insert::colort(*$2); delete($2);		}
				;

pencolort	: PENCOLOR_T common	{ Insert::pencolort(*$2); delete($2);	}
				;

deptht		: DEPTH_T common		{ Insert::deptht(*$2); delete($2);		}
				;

textt			: TEXT_T common		{ Insert::textt(*$2); delete($2);		}
				;

otheratom	: TERM common			{ delete($1); delete($2);					}
				;

common		: '(' terms ')' '.'				{ $$ = $2;	}

terms			: terms ',' term	{ $$ = $1; $$->push_back(*$3); delete($3);		}
				| term				{ $$ = new vector<string>(1,*$1); delete($1);	}
				;

term			: TERM	{ $$ = $1;	}
				;

%%

void parseerror(const char* s) {
	Error::error();
	cerr << s << endl;
}
