#ifndef IDP_SHAPE_H
#define IDP_SHAPE_H

#include <vector>
#include <QPoint>
#include <QRect>

using namespace std;

enum STYPE { POLY, ELLI };

class Shape {

	public:
		
		// Inspectors
		virtual	int	minx()	= 0; 
		virtual	int	miny()	= 0; 
		virtual	int	maxx()	= 0; 
		virtual	int	maxy()	= 0; 
		virtual	STYPE	type()	const = 0;
		virtual	QRect brect()	= 0;

};

class PolyShape : public Shape {
	
	private:
		vector<QPoint> _points;

	public:

		// Constructor
		PolyShape(vector<QPoint>& vq); 

		// Inspectors
		vector<QPoint> points()			const	{ return _points;				}
		int				nrPoints()		const	{ return _points.size();	}
		QPoint			point(int n)	const { return _points[n];			}
		int				minx()			; 
		int				miny()			; 
		int				maxx()			; 
		int				maxy()			; 
		STYPE				type()			const		{ return POLY;		}
		QRect				brect()			;
		
};

class ElliShape : public Shape {

	private:
		int _width;
		int _height;

	public:
		
		// Constructor
		ElliShape(int s1, int s2) :  _width(s1), _height(s2) { }

		// Inspectors
		int		width()	const { return _width;	}
		int		height()	const { return _height;	}
		int		minx()	 { return 0;			}
		int		miny()	 { return 0;			}
		int		maxx()	 { return _width;	}
		int		maxy()	 { return _height;	}
		STYPE		type()	const	{ return ELLI;		}
		QRect		brect()	;

};

#endif 
