# requires python 2.6

import sys
from bioasp import asp

def is_number(s):
  try:
    int(s)
    return True
  except ValueError:
    return False 

def debug(s):
  import inspect
  print >>sys.stderr, "DBG %03d: %s" % \
    (inspect.currentframe().f_back.f_lineno,s)


maximum_models = 50

#dictionary_asp='common.dict'
dictionary_asp='common_dict.asp'

#input_asp='testsuite/instances/john_gave_ann_the_dog_and_marcel_the_cat/input'
input_asp='examples_common_dict/the_dog_bit_john.asp'

parser_asp='ccg.asp'
ccg2idpdraw_asp = 'ccg2idpdraw.asp'


maxstep_options = '--const maxsteps=8'

# the clasp/gringo coming with bioasp has some deficiencies (beta version)
# something like the next line allows to use a script
# that simply contains "clasp --eq=0 $*"
#clasp_bin = '/home/ps/_work/kbs/aspccg/claspwrapper.sh'



clasp_bin = './claspwrapper.sh'
#'./bioasp/bin/clasp'
#
gringo_bin ='./gringowrapper.sh'
#'./bioasp/bin/gringo'
#

idpdraw_bin = './IDPDraw/code/IDPDraw'

if len(sys.argv) > 1:
  i=1
  while i<len(sys.argv):    
    if is_number(sys.argv[i]):
      maxstep_options = '--const maxsteps='+sys.argv[i]
    elif  sys.argv[i].find("-p=")!=-1:
      parser_asp=sys.argv[i][sys.argv[i].find("-p=")+3:]
    elif  sys.argv[i].find("-d=")!=-1:
      dictionary_asp=sys.argv[i][sys.argv[i].find("-d=")+3:]
    elif  sys.argv[i].find('-h')!=-1:
      print "OPTIONS:"
      print "-p=<parser.clingo> -- specifies parser encoding to use"
      print "-d=<dictionary.asp> -- specifies parser encoding to use"
      print "integer -- specifies maxsteps"
      print "<instance_file> -- specifies instance to parse"
      print "-h -- prints these options"
      sys.exit()
    else:
      input_asp = sys.argv[i]
    i=i+1


idpdrawfile = input_asp + '.idpdraw'

parse_programs = [ parser_asp, dictionary_asp, input_asp ]

# first try to parse with strict result
debug("parsing with strict goal ...")
solver_parsing = asp.GringoClasp(gringo_options=maxstep_options,
  clasp_bin=clasp_bin, gringo_bin=gringo_bin)
parse_additional = 'strict_parse. use_category_class(0..4).'
parse_models = solver_parsing.run(
  parse_programs, additionalProgramText=parse_additional,
  nmodels=maximum_models, collapseAtoms=True,collapseTerms=True)
#debug("strict parse models = %s" % (parse_models,))

if len(parse_models) == 0:
  # strict parsing was not successful, now try with optimization
  debug("parsing with optimization ...")
  solver_optparsing = asp.GringoClaspOpt(gringo_options=maxstep_options,
    clasp_bin=clasp_bin, gringo_bin=gringo_bin)
  optparse_additional = '#minimize { position_status_open(P,maxsteps) }. use_category_class(0..4).'
  optparse_models = solver_optparsing.run(
    parse_programs, additionalProgramText=optparse_additional,
    nmodels=maximum_models, collapseAtoms=True,collapseTerms=True)
  #debug("optimization parse models = %s" % (optparse_models,))
  if optparse_models == None:
    raise RuntimeError("unsatisfiable even with optimization!")
  parse_models = optparse_models[1]
#debug("parse models (strict or opt): %s" % parse_models)

debug("got %d parse models (maximum set to %d)" % (len(parse_models), maximum_models))
# calculate idpdraw display answer sets
debug("calculating idpdraw answer sets")
solver_idpdraw = asp.GringoClasp(
  clasp_bin=clasp_bin, gringo_bin=gringo_bin)
idpdraw_models = []
for parse_model in parse_models:
  #debug(parse_model)
  idpdraw_localmodels = solver_idpdraw.run(
    [ccg2idpdraw_asp, parse_model.to_file()],
    nmodels=2, collapseAtoms=False,collapseTerms=False)
  if len(idpdraw_localmodels) != 1:
    raise RuntimeError("expected exactly 1 idpdraw model for parser model" +
      " got %s" % (idpdraw_models,))
  idpdraw_models.append(idpdraw_localmodels[0])
  #debug("idpdraw model = %s" % (idpdraw_localmodels[0],))

def formatCategoryInternal(term, outerBrackets=True):
  if term.__class__ != asp.Term:
    # strings are stripped of their '"'
    # integers are simply returned as strings
    return str(term).strip('"')

  slash = None
  if term.pred() == 'lfunc':
    slash = '\\'
  elif term.pred() == 'rfunc':
    slash = '/'
  else:
    raise RuntimeError("cannot convert term %s (should be some category)" % term)
  assert(slash != None)

  args = map(lambda x: formatCategoryInternal(x), term.args())
  assert(len(args) == 2)
  if outerBrackets:
    return '(%s%s%s)' % (args[0], slash, args[1])
  else:
    return '%s%s%s' % (args[0], slash, args[1])

def formatCategory(term):
  return '"' + formatCategoryInternal(term,False) + '"'

def convertModelsIntoIDPDrawFile(models, ofname):
  overallmodel = asp.TermSet([])
  counter = 0
  for model in models:
    # first remove everything that does not start with idpd_
    for atom in model:
      if not atom.pred().startswith('idpd_'):
        continue
      atom.args().insert(0,counter)
      if atom.pred() == 'idpd_text_t':
        lastarg = atom.args()[-1]
        #debug(lastarg)
        if lastarg.__class__ == asp.Term:
          atom.args()[-1] = formatCategory(lastarg)
      #debug(str(atom))
      overallmodel.add(atom) 
    counter += 1
  overallmodel.to_file(ofname)

# convert idpdraw results to make them idpdraw-display-able
# also add timestamps
convertModelsIntoIDPDrawFile(idpdraw_models, idpdrawfile)

# display with idpdraw
import subprocess
proc = subprocess.Popen([idpdraw_bin, idpdrawfile])
proc.wait()
