# requires python 2.6

import sys
import tempfile
import subprocess
import re

def debug(s):
  import inspect
  print >>sys.stderr, "DBG %03d: %s" % \
    (inspect.currentframe().f_back.f_lineno,s)

input_sentence = 'John walks the dog .'
if len(sys.argv) > 1:
  i=1
  while i<len(sys.argv):    
    if sys.argv[i].startswith("-s="):
      input_sentence = sys.argv[i][3:]
    elif  sys.argv[i].find("-p=")!=-1:
      parser_asp=sys.argv[i][sys.argv[i].find("-p=")+3:]
    else: # sys.argv[i] == "-h":
      print "OPTIONS:"
      print "-s=<sentence to supertag"
      #print "-p=<*parser.clingo> -- specifies parser encoding to use"
      print "-h -- prints these options"
      sys.exit()
    i += 1

# will create classes numbered from 0 to k-1, with 0 containing the most
# probable categories and k-1 containing the least probable categories
beta_values = [ 0.075, 0.03, 0.01, 0.005, 0.001 ]

candc_bindir="candc/candc-1.00/bin/"
candc_modelsdir="candc/models/"

candc_postagger = candc_bindir + 'pos'
candc_posmodels = candc_modelsdir + 'pos'

candc_supertagger = candc_bindir + 'msuper'
candc_supermodels = candc_modelsdir + 'super'

import os
if not os.access(candc_postagger, os.X_OK) or \
    not os.access(candc_posmodels, os.X_OK):
  raise RuntimeError("please un-tar-gz candc and models!")

debug("postagging sentence '%s'" % (input_sentence,))
postagger_process = subprocess.Popen(
  [candc_postagger, '--model', candc_posmodels],
  stderr=subprocess.PIPE, stdout=subprocess.PIPE, stdin=subprocess.PIPE)
postagger_result, postagger_error = postagger_process.communicate(input_sentence)
debug("from postagger got result '%s'" % (postagger_result.strip(),))
if postagger_process.returncode != 0:
  debug("from postagger got error '%s'" % (postagger_error,))

def categoryHR2Function(category_human_readable):
  '''
  converts human readable category string into functional notation
  e.g., "NP[dcl]/(S\NP)" is converted into rfunc("NP[dcl]",lfunc("S","NP"))
  '''
  def findSlashIndex(s):
    #debug(s)
    depth = 0
    for at in range(0,len(s)):
      char = s[at]
      if char == '(':
        depth += 1
      elif char == ')':
        depth -= 1
      elif (char == '/' or char == '\\') and depth == 0:
        return at
    return None
  si = findSlashIndex(category_human_readable)
  #debug(si)
  if si == None:
    # string category or bracketed expression
    if category_human_readable[0] == '(' and category_human_readable[-1] == ')':
      return categoryHR2Function(category_human_readable[1:-1])
    else:
      return '"' + category_human_readable + '"'
  else:
    # slash
    left = categoryHR2Function(category_human_readable[:si])
    right = categoryHR2Function(category_human_readable[si+1:])
    #debug("in slashed string '%s': '%s' <slash> '%s'" % \
    #  (category_human_readable, category_human_readable[:si], category_human_readable[si+1:]))
    #debug("left='%s' right='%s'" % (left, right))
    function = None
    if category_human_readable[si] == '/':
      function = 'rfunc'
    else:
      function = 'lfunc'
    assert(function != None)
    # forward slash
    return '%s(%s,%s)' % (function, left, right)
  assert(False)

def cleanFilename(fname):
  return re.sub(r'[,./\\\']', '_', fname)

def beta2Class(beta_values, beta):
  #debug("calculating class index for beta %0.3f" % (beta,))
  for cls in range(0,len(beta_values)):
    #debug("%0.3f vs %0.3f" % (beta, beta_values[cls]))
    if beta >= beta_values[cls]:
      return cls
  # fallback to last value (float comparison is tricky for 0.001)
  return len(beta_values)-1

# make sure beta_values is sorted
tmpbeta = beta_values
tmpbeta.sort()
tmpbeta.reverse()
assert(tmpbeta == beta_values)

sentence_identifier = None
# last beta value = smallest
minbeta = beta_values[-1]

debug("supertagging sentence '%s' with beta %0.3f" % (input_sentence, minbeta))
supertagger_process = subprocess.Popen(
  [candc_supertagger, '--model', candc_supermodels, '--beta', "%0.3f" % (minbeta,)],
  stderr=subprocess.PIPE, stdout=subprocess.PIPE, stdin=subprocess.PIPE)
supertagger_result, supertagger_error = supertagger_process.communicate(postagger_result)
#debug("from supertagger got result '%s'" % (supertagger_result,))
#debug("from supertagger got error '%s'" % (supertagger_error,))
# parse supertagger result
words = []
for word in supertagger_result.split('\n'):
  wordtokens = word.split('\t')
  if len(wordtokens) < 2:
    continue
  #debug("position %d wordtokens = %s" % (len(words), str(wordtokens)))
  fieldcount = int(wordtokens[2])
  assert(len(wordtokens) == 3 + fieldcount*2)
  # build list of categories for each word
  categories = []
  for i in range(0,fieldcount):
    categories.append(wordtokens[3+i*2:3+i*2+2])
  thisword = (wordtokens[0], categories)
  #debug(thisword)
  words.append(thisword)

# create sentence identifier
new_sentence_identifier = cleanFilename(
  '_'.join(map(lambda x: x[0], words)))
if sentence_identifier == None:
  sentence_identifier = new_sentence_identifier
assert(sentence_identifier == new_sentence_identifier)

# write sentence output file
sentencefilename = 'examples_supertagged/chunked_%s.asp' % \
  (sentence_identifier,)
debug("writing " + sentencefilename)
sentencefile = open(sentencefilename, 'w+')
position_int = 0
for word in words:
  word_string = word[0]
  sentencefile.write('word_at("%s",%d).\n' % (word_string, position_int))
  if position_int != 0:
    sentencefile.write('adjacent(%d,%d).\n' % (position_int-1, position_int))
  position_int += 1
sentencefile.close() 

# write sentence+tagged category output for this sentence
taggedfilename = 'examples_supertagged/supertagged_%s.asp' % \
  (sentence_identifier,)
#debug("writing " + taggedfilename)
taggedfile = open(taggedfilename, 'w+')
position_int = 0
# write beta class
for cls in range(0,len(beta_values)):
  taggedfile.write('tag_probability_class(%d,"%0.3f").\n' % (cls, beta_values[cls]))
for word in words:
  word_string = word[0]
  word_cats = word[1]
  taggedfile.write('word_at("%s",%d).\n' % (word_string, position_int))
  if position_int != 0:
    taggedfile.write('adjacent(%d,%d).\n' % (position_int-1, position_int))
  for cat in word_cats:
    catstr = cat[0]
    catprob = float(cat[1])
    cat_as_function = categoryHR2Function(catstr)
    probclass = beta2Class(beta_values, catprob)
    debug('category for position %d = %s with probability %0.3f = class %d' % \
      (position_int, catstr, catprob, probclass))
    taggedfile.write('category_tag(%d,%s,%d).\n' % \
      (position_int, cat_as_function, probclass))
    stripped_cat_as_function = re.sub(r'\[.+?\]', '', cat_as_function)
    taggedfile.write('category_tag_nofeatures(%d,%s,%d).\n' % \
      (position_int, stripped_cat_as_function, probclass))
  position_int += 1
taggedfile.close() 

